import os
import json
from typing import Optional, List, Dict, Any, Union

from mcp.server.models import InitializationOptions
import mcp.types as types
from mcp.server import NotificationOptions, Server
import mcp.server.stdio
from atlassian import Confluence

# Initialize Confluence client
CONFLUENCE_URL = os.getenv("CONFLUENCE_URL")
CONFLUENCE_USERNAME = os.getenv("CONFLUENCE_USERNAME")
CONFLUENCE_API_TOKEN = os.getenv("CONFLUENCE_API_TOKEN")

if not all([CONFLUENCE_URL, CONFLUENCE_USERNAME, CONFLUENCE_API_TOKEN]):
    raise ValueError("Missing required Confluence environment variables")

confluence = Confluence(
    url=CONFLUENCE_URL,
    username=CONFLUENCE_USERNAME,
    password=CONFLUENCE_API_TOKEN,
    cloud=True  # Assuming cloud instance, set to False for server
)

server = Server("hc-mcp-confluence")

@server.list_tools()
async def handle_list_tools() -> list[types.Tool]:
    """List available Confluence tools."""
    return [
        types.Tool(
            name="get_page",
            description="Get content of a specific Confluence page",
            inputSchema={
                "type": "object",
                "properties": {
                    "page_id": {
                        "type": "string",
                        "description": "Confluence page ID"
                    },
                    "expand": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "List of properties to expand"
                    }
                },
                "required": ["page_id"]
            }
        ),
        types.Tool(
            name="create_page",
            description="Create a new Confluence page",
            inputSchema={
                "type": "object",
                "properties": {
                    "space_key": {
                        "type": "string",
                        "description": "Key of the space where the page will be created"
                    },
                    "title": {
                        "type": "string",
                        "description": "Title of the page"
                    },
                    "body": {
                        "type": "string",
                        "description": "Content of the page in storage format (XHTML)"
                    },
                    "parent_id": {
                        "type": "string",
                        "description": "ID of the parent page (optional)"
                    },
                    "labels": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "List of labels to add to the page"
                    }
                },
                "required": ["space_key", "title", "body"]
            }
        ),
        types.Tool(
            name="update_page",
            description="Update an existing Confluence page",
            inputSchema={
                "type": "object",
                "properties": {
                    "page_id": {
                        "type": "string",
                        "description": "ID of the page to update"
                    },
                    "title": {
                        "type": "string",
                        "description": "New title for the page"
                    },
                    "body": {
                        "type": "string",
                        "description": "New content for the page in storage format (XHTML)"
                    },
                    "version_comment": {
                        "type": "string",
                        "description": "Comment for the new version"
                    }
                },
                "required": ["page_id", "body"]
            }
        ),
        types.Tool(
            name="delete_page",
            description="Delete a Confluence page",
            inputSchema={
                "type": "object",
                "properties": {
                    "page_id": {
                        "type": "string",
                        "description": "ID of the page to delete"
                    }
                },
                "required": ["page_id"]
            }
        ),
        types.Tool(
            name="search_content",
            description="Search Confluence content using CQL",
            inputSchema={
                "type": "object",
                "properties": {
                    "cql": {
                        "type": "string",
                        "description": "Confluence Query Language (CQL) search string"
                    },
                    "limit": {
                        "type": "integer",
                        "description": "Maximum number of results to return",
                        "default": 25
                    },
                    "expand": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "List of properties to expand in the results"
                    }
                },
                "required": ["cql"]
            }
        ),
        types.Tool(
            name="get_space",
            description="Get details about a Confluence space",
            inputSchema={
                "type": "object",
                "properties": {
                    "space_key": {
                        "type": "string",
                        "description": "Key of the space"
                    },
                    "expand": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "List of properties to expand"
                    }
                },
                "required": ["space_key"]
            }
        ),
        types.Tool(
            name="list_spaces",
            description="List all Confluence spaces",
            inputSchema={
                "type": "object",
                "properties": {
                    "limit": {
                        "type": "integer",
                        "description": "Maximum number of spaces to return",
                        "default": 25
                    },
                    "expand": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "List of properties to expand"
                    }
                }
            }
        ),
        types.Tool(
            name="add_page_labels",
            description="Add labels to a Confluence page",
            inputSchema={
                "type": "object",
                "properties": {
                    "page_id": {
                        "type": "string",
                        "description": "ID of the page"
                    },
                    "labels": {
                        "type": "array",
                        "items": {"type": "string"},
                        "description": "List of labels to add"
                    }
                },
                "required": ["page_id", "labels"]
            }
        ),
        types.Tool(
            name="get_page_labels",
            description="Get labels of a Confluence page",
            inputSchema={
                "type": "object",
                "properties": {
                    "page_id": {
                        "type": "string",
                        "description": "ID of the page"
                    }
                },
                "required": ["page_id"]
            }
        ),
        types.Tool(
            name="get_page_attachments",
            description="Get attachments of a Confluence page",
            inputSchema={
                "type": "object",
                "properties": {
                    "page_id": {
                        "type": "string",
                        "description": "ID of the page"
                    }
                },
                "required": ["page_id"]
            }
        )
    ]

@server.call_tool()
async def handle_call_tool(
    name: str, arguments: dict | None
) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
    """Handle Confluence tool execution requests."""
    try:
        if not arguments:
            raise ValueError("Arguments are required")

        if name == "get_page":
            page_id = arguments.get("page_id")
            expand = arguments.get("expand", ["body.storage", "version"])
            
            if not page_id:
                raise ValueError("page_id is required")
                
            page = confluence.get_page_by_id(
                page_id,
                expand=",".join(expand)
            )
            
            return [types.TextContent(
                type="text",
                text=json.dumps(page, indent=2)
            )]

        elif name == "create_page":
            space_key = arguments.get("space_key")
            title = arguments.get("title")
            body = arguments.get("body")
            parent_id = arguments.get("parent_id")
            labels = arguments.get("labels", [])
            
            if not all([space_key, title, body]):
                raise ValueError("space_key, title, and body are required")
            
            page = confluence.create_page(
                space=space_key,
                title=title,
                body=body,
                parent_id=parent_id,
                representation="storage"
            )
            
            if labels:
                confluence.set_page_labels(
                    page_id=page["id"],
                    labels=labels
                )
            
            return [types.TextContent(
                type="text",
                text=json.dumps(page, indent=2)
            )]

        elif name == "update_page":
            page_id = arguments.get("page_id")
            body = arguments.get("body")
            title = arguments.get("title")
            version_comment = arguments.get("version_comment")
            
            if not all([page_id, body]):
                raise ValueError("page_id and body are required")
            
            # Get current page info
            page = confluence.get_page_by_id(
                page_id,
                expand="version"
            )
            
            # Update the page
            updated_page = confluence.update_page(
                page_id=page_id,
                title=title or page["title"],
                body=body,
                parent_id=page.get("parent", {}).get("id"),
                version_comment=version_comment,
                representation="storage"
            )
            
            return [types.TextContent(
                type="text",
                text=json.dumps(updated_page, indent=2)
            )]

        elif name == "delete_page":
            page_id = arguments.get("page_id")
            
            if not page_id:
                raise ValueError("page_id is required")
            
            confluence.delete_page(page_id)
            
            return [types.TextContent(
                type="text",
                text=json.dumps({"message": "Page deleted successfully"}, indent=2)
            )]

        elif name == "search_content":
            cql = arguments.get("cql")
            limit = arguments.get("limit", 25)
            expand = arguments.get("expand", ["body.storage", "version"])
            
            if not cql:
                raise ValueError("cql is required")
            
            results = confluence.cql(
                cql,
                limit=limit,
                expand=",".join(expand)
            )
            
            return [types.TextContent(
                type="text",
                text=json.dumps(results, indent=2)
            )]

        elif name == "get_space":
            space_key = arguments.get("space_key")
            expand = arguments.get("expand", ["description"])
            
            if not space_key:
                raise ValueError("space_key is required")
            
            space = confluence.get_space(
                space_key,
                expand=",".join(expand)
            )
            
            return [types.TextContent(
                type="text",
                text=json.dumps(space, indent=2)
            )]

        elif name == "list_spaces":
            limit = arguments.get("limit", 25)
            expand = arguments.get("expand", ["description"])
            
            spaces = confluence.get_all_spaces(
                limit=limit,
                expand=",".join(expand)
            )
            
            return [types.TextContent(
                type="text",
                text=json.dumps(spaces, indent=2)
            )]

        elif name == "add_page_labels":
            page_id = arguments.get("page_id")
            labels = arguments.get("labels", [])
            
            if not all([page_id, labels]):
                raise ValueError("page_id and labels are required")
            
            result = confluence.set_page_labels(
                page_id=page_id,
                labels=labels
            )
            
            return [types.TextContent(
                type="text",
                text=json.dumps(result, indent=2)
            )]

        elif name == "get_page_labels":
            page_id = arguments.get("page_id")
            
            if not page_id:
                raise ValueError("page_id is required")
            
            labels = confluence.get_page_labels(page_id)
            
            return [types.TextContent(
                type="text",
                text=json.dumps(labels, indent=2)
            )]

        elif name == "get_page_attachments":
            page_id = arguments.get("page_id")
            
            if not page_id:
                raise ValueError("page_id is required")
            
            attachments = confluence.get_attachments_from_content(page_id)
            
            return [types.TextContent(
                type="text",
                text=json.dumps(attachments, indent=2)
            )]

        else:
            raise ValueError(f"Unknown tool: {name}")

    except Exception as e:
        return [types.TextContent(
            type="text",
            text=json.dumps({
                "error": str(e),
                "details": {
                    "name": e.__class__.__name__,
                    "message": str(e),
                    "tool": name,
                    "arguments": arguments
                }
            }, indent=2),
            isError=True
        )]

async def main():
    """Run the Confluence MCP server."""
    async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
        await server.run(
            read_stream,
            write_stream,
            InitializationOptions(
                server_name="hc-mcp-confluence",
                server_version="0.1.0",
                capabilities=server.get_capabilities(
                    notification_options=NotificationOptions(),
                    experimental_capabilities={},
                ),
            ),
        )
